@@ -13,7 +13,8 @@ from time import sleep
 import requests
 
 from .version import __version__
-from .utils import backoff_retry, print_with_time
+from .utils import *
+from .random_ua import ua
 
 
 class Badminton():
@@ -25,10 +26,9 @@ class Badminton():
         self.headers = {
             'Accept': '*/*',
             'Accept-Encoding': 'gzip, deflate, br',
-            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 \
-            (KHTML, like Gecko) Chrome/87.0.4280.66 Safari/537.36",
+            "User-Agent": ua(),
         }
-        self.stocks, self.selects, self.results = [], [], []
+        self.stocks, self.selects, self.unpaid = [], [], []
         self.capt, self.imbyte = None, None
         self.max_times = max_times
         self.token = token
@@ -71,6 +71,79 @@ class Badminton():
         print_with_time(f'每个时间段最大预订场地数: {self.stock_number}')
 
 
+    def login(self):
+        """登录
+        """
+        login_url = "https://cas.sysu.edu.cn/cas/login"
+        captcha_url = "https://cas.sysu.edu.cn/cas/captcha.jsp"
+        login_form = {"username": "",
+                      "password": "",
+                      "captcha": "",
+                      "_eventId": "submit",
+                      "geolocation": "",
+                      }
+        login_times = 0
+
+        while login_times < self.max_times:
+            login_times += 1
+            r = self.session.get(login_url, headers=self.headers)
+            if 'Log In Successful' in r.text:
+                print_with_time('登录有效')
+                return True
+            print_with_time('开始登录')
+            self.imbyte = self.session.get(captcha_url).content
+            self.capt = recognize(self.imbyte, self.token, self.max_times)
+            login_form['captcha'] = self.capt
+            login_form['execution'] = findall(
+                'name="execution" value="(.*?)"', r.text)[0]
+            login_form["username"] = self.netid
+            login_form["password"] = self.password
+
+            r = self.session.post(
+                login_url, headers=self.headers, data=login_form)
+            if 'success' in r.text:
+                print_with_time('登录成功')
+                return True
+            if 'credential' in r.text:
+                print_with_time('用户名或密码错误')
+                sys.exit()
+
+            print_with_time(f'验证码错误，第 {login_times} 次重试')
+            sleep(1)
+
+        print_with_time('登录重试次数太多，终止')
+        sys.exit()
+
+
+    def wait(self):
+        """系统正常开启时间为 00:00-22:00，选择合适等待策略
+        """
+        now_time = datetime.now()
+
+        if now_time > self.end_datetime:
+            print_with_time(f'预订日期已经过去，你可以穿越回 {self.end_datetime} 之前，再见')
+            sys.exit()
+
+        if now_time > self.begin_datetime:
+            if now_time.hour < 22:
+                print_with_time(f'当前时间 {datetime.now()}，开始预订')
+                return
+            self.begin_datetime += timedelta(days=1)
+            print_with_time('预订系统关闭，等到明天开始预订')
+
+        print_with_time(f'未到可预订时间，等到 {self.begin_datetime} 开始预订')
+        if (self.begin_datetime - datetime.now()).days > 0:
+            print_with_time('!!!警告!!!')
+            print_with_time('等待时间超过一天，建议等到明天再来预订!!!')
+        print_with_time(f'{self.begin_datetime - datetime.now()} 后开始预订')
+        sleep((self.begin_datetime - datetime.now()).seconds - 120)
+        print_with_time('检查登录状态')
+        if self.login():
+            print_with_time(f'{self.begin_datetime - datetime.now()} 后开始预订')
+            sleep((self.begin_datetime - datetime.now()).seconds - 3)
+            return
+
+
     def get_stocks(self):
         """获取指定日期可预订的场次列表
         """
@@ -96,7 +169,7 @@ class Badminton():
                 dic = {}
                 if stock['status'] == 1:
                     for keys in ['id', 'stockid']:
-                        dic[keys] = stock[keys]
+                        dic[keys] = str(stock[keys])
                     for keys in ['price', 's_date']:
                         dic[keys] = stock['stock'][keys]
                     dic['sname'] = int(stock['sname'][2:])
@@ -142,128 +215,30 @@ class Badminton():
             print_with_time(f"{stock['id']} 场地{stock['sname']} {stock['time_full']}")
 
 
-    def _recognize(self):
-        """调用 cascaptcha 在线识别验证码
-        """
-        headers = {'Connection': 'Keep-Alive',
-                'Authorization': self.token,
-                'User-Agent': 'Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 5.1; Trident/4.0)'}
-        files = {'imgfile': ('captcha.jpg', self.imbyte)}
-        recg_times = 0
-        while recg_times < self.max_times:
-            r = requests.post('https://cascaptcha.vercel.app/api', files=files, headers=headers)
-            arrstr = json.loads(r.text)
-            if arrstr["success"]:
-                print_with_time(f'验证码识别成功: {arrstr["captcha"]}')
-                self.capt = arrstr["captcha"]
-                return True
-            recg_times += 1
-            print_with_time(f'识别失败，第 {recg_times} 次重试')
-            sleep(1)
-
-        with open('capt.jpg', 'wb') as f:
-            f.write(self.imbyte)
-        self.capt = input('验证码识别失败，请手动打开 capt.jpg 识别后输入\n')
-
-
-    def login(self):
-        """登录
-        """
-        login_url = "https://cas.sysu.edu.cn/cas/login"
-        captcha_url = "https://cas.sysu.edu.cn/cas/captcha.jsp"
-        login_form = {"username": "",
-                      "password": "",
-                      "captcha": "",
-                      "_eventId": "submit",
-                      "geolocation": "",
-                      }
-        login_times = 0
-        print_with_time('开始登录')
-
-        while login_times < self.max_times:
-            login_times += 1
-            r = self.session.get(login_url, headers=self.headers)
-            if 'Log In Successful' in r.text:
-                print_with_time('登录成功')
-                return True
-            self.imbyte = self.session.get(captcha_url).content
-            self._recognize()
-            login_form['captcha'] = self.capt
-            login_form['execution'] = findall('name="execution" value="(.*?)"', r.text)[0]
-            login_form["username"] = self.netid
-            login_form["password"] = self.password
-
-            r = self.session.post(login_url, headers=self.headers, data=login_form)
-            if 'success' in r.text:
-                print_with_time('登录成功')
-                return True
-            if 'credential' in r.text:
-                print_with_time('用户名或密码错误')
-                sys.exit()
-
-            print_with_time(f'验证码错误，第 {login_times} 次重试')
-            sleep(1)
-
-        print_with_time('登录重试次数太多，终止')
-        sys.exit()
-
-
-    def wait(self):
-        """系统正常开启时间为 00:00-22:00，选择合适等待策略
-        """
-        now_time = datetime.now()
-
-        if now_time > self.end_datetime:
-            print_with_time(f'预订日期已经过去，你可以穿越回 {self.end_datetime} 之前，再见')
-            sys.exit()
-
-        if now_time > self.begin_datetime:
-            if now_time.hour < 22:
-                print_with_time(f'当前时间 {datetime.now()}，开始预订')
-                return
-            self.begin_datetime += timedelta(days=1)
-            print_with_time('预订系统关闭，等到明天开始预订')
-
-        print_with_time(f'未到可预订时间，等到 {self.begin_datetime} 开始预订')
-        if (self.begin_datetime - datetime.now()).days > 0:
-            print_with_time('!!!警告!!!')
-            print_with_time('等待时间超过一天，建议等到明天再来预订!!!')
-        print_with_time(f'{self.begin_datetime - datetime.now()} 后开始预订')
-        sleep((self.begin_datetime - datetime.now()).seconds - 120)
-        print_with_time('检查登录状态')
-        if self.login():
-            print_with_time(f'{self.begin_datetime - datetime.now()} 后开始预订')
-            sleep((self.begin_datetime - datetime.now()).seconds - 3)
-            return
-
-
     def book(self):
         """预订场地
         """
         stock_index = 0
         book_times = 0
-        self.session.get('https://gym.sysu.edu.cn/yyuser/personal.html', headers=self.headers)
-        while len(self.results) < 2 and book_times < 10:
+        personal_url = 'https://gym.sysu.edu.cn/yyuser/personal.html'
+        book_url = 'https://gym.sysu.edu.cn/order/book.html'
+        print_with_time('开始预订')
+        personal_page = self.session.get(personal_url, headers=self.headers)
+        unpaid_orders = find_unpaid_orders(personal_page.text)
+        while len(self.unpaid) < 2 and book_times < 10:
             book_times += 1
             try:
                 stock = self.selects[stock_index]
             except IndexError:
                 print_with_time('可以预订的都试过了')
                 break
-            param = ('{"activityPrice":0,"activityStr":null,"address":null,"dates":null,'
-                     '"extend":null,"flag":"0","isbookall":"0","isfreeman":"0","istimes":"1",'
-                     '"merccode":null,"order":null,"orderfrom":null,"remark":null,"serviceid":'
-                     'null,"shoppingcart":"0","sno":null,"stock":{"%s":"1"},"stockdetail":'
-                     '{"%s":"%s"},"stockdetailids":"%s","subscriber":"0",'
-                     '"time_detailnames":null,"userBean":null}') \
-                % (stock['stockid'], stock['stockid'], stock['id'], stock['id'])
-            data = {"param": param, "json": "true"}
-            r = self.session.post("https://gym.sysu.edu.cn/order/book.html",
-                                  headers=self.headers, data=data)
+            data = {"param": book_param(stock), "json": "true"}
             try:
+                r = self.session.post(book_url, headers=self.headers, data=data)
                 return_data = json.loads(r.text)
             except JSONDecodeError:
                 print_with_time(f'预订失败，第 {book_times} 次重试')
+                print_with_time(r.text)
                 sleep(10)
                 continue
 
@@ -271,13 +246,11 @@ class Badminton():
                 print_with_time(f'预订成功：场地 {stock["sname"]} {stock["time_full"]} '
                                 f'订单号 {return_data["object"]["orderid"]}，待支付')
                 stock_index += 1
-                self.results.append([return_data['object']['orderid'],
-                                    stock["sname"], stock["s_date"], stock["time_no"]])
+                self.unpaid.append(return_data['object']['orderid'])
 
             elif return_data['message'] == 'USERNOTLOGINYET':
-                print_with_time('JSESSIONID 过期了，重新登录')
-                sleep(1)
-                self.login()
+                personal_page = self.session.get(personal_url, headers=self.headers)
+                unpaid_orders = find_unpaid_orders(personal_page.text)
                 continue
 
             elif '已被预订' in return_data['message']:
@@ -290,13 +263,16 @@ class Badminton():
                 sleep(10)
                 continue
 
-        if len(self.results) == 0:
+        if unpaid_orders:
+            print_with_time(f'发现未支付订单：{unpaid_orders}，之后一起支付')
+            self.unpaid = list(set(self.unpaid + unpaid_orders))
+        if not self.unpaid:
             print_with_time('预订失败，重试次数过多，退出')
             sys.exit()
         if not self.auto_pay:
-            print_with_time(f'预订成功 {len(self.results)} 个场地，请自行支付')
+            print_with_time(f'预订成功 {len(self.unpaid)} 个场地，请自行支付')
             sys.exit()
-        print_with_time(f'预订成功 {len(self.results)} 个场地，10 秒后自动支付订单')
+        print_with_time(f'预订成功 {len(self.unpaid)} 个场地，10 秒后自动支付订单')
         sleep(10)
 
 
@@ -304,17 +280,17 @@ class Badminton():
         """支付订单
         """
         pay_times = 0
-        while pay_times < self.max_times and len(self.results) != 0:
+        while pay_times < self.max_times and len(self.unpaid) != 0:
             pay_times += 1
-            result = self.results[0]
-            print_with_time('支付订单 ' + str(result[0]))
-            data1 = {"orderid": result[0], "payid": 2}
+            order_pay = self.unpaid[0]
+            print_with_time('支付订单 ' + str(order_pay))
+            data1 = {"orderid": order_pay, "payid": 2}
             headers = self.headers
-            headers['Referer'] = f'https://gym.sysu.edu.cn/pay/show.html?id={result[0]}'
+            headers['Referer'] = f'https://gym.sysu.edu.cn/pay/show.html?id={order_pay}'
             data2 = {'param': json.dumps(
-                {"payid": 2, "orderid": str(result[0]), "ctypeindex": 0})}
+                {"payid": 2, "orderid": str(order_pay), "ctypeindex": 0})}
 
-            r = self.session.get("https://gym.sysu.edu.cn/pay/account/showpay.html",
+            self.session.get("https://gym.sysu.edu.cn/pay/account/showpay.html",
                                  headers=headers, data=data1)
             r = self.session.post("https://gym.sysu.edu.cn/pay/account/topay.html",
                                   headers=headers, data=data2)
@@ -326,13 +302,13 @@ class Badminton():
                 self.login()
                 continue
             if return_data["result"] == "1":
-                print_with_time(f'{str(result[0])} 支付成功')
-                self.results.remove(result)
+                print_with_time(f'{str(order_pay)} 支付成功')
+                self.unpaid.remove(order_pay)
             else:
                 print_with_time(f'支付结果: {return_data["message"]}，重试')
                 sleep(5)
 
-        if len(self.results) != 0:
+        if len(self.unpaid) != 0:
             print_with_time('可能有未成功支付的订单，请手动支付')
 
 
